/* module EID210 */

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include "qsm.h"

#include "eid210.h"
#include "feux.h"

#ifndef RAMEND
#define RAMEND (caddr_t)0x0081FFFF
#endif

#ifndef NULL
#define NULL 0
#endif

extern caddr_t _end;

#undef errno
int errno;

caddr_t sbrk(int nbytes)
{
static caddr_t heap_ptr=NULL;
caddr_t base;
if (heap_ptr==NULL)
	heap_ptr=(caddr_t)&_end;
if (RAMEND-heap_ptr>=0)
	{
	base=heap_ptr;
	heap_ptr+=nbytes;
	return base;
	}
	else
	{
	errno=ENOMEM;
	return ((caddr_t)-1);
	}
}

int isatty(int fd)
{
	return 1;
}

int close(int fd)
{
	return 0;
}

off_t lseek(int fd,off_t offset)
{
	errno=ESPIPE;
	return (offset-1);
}

int fstat(int fd,struct stat *buf)
{
	errno=EIO;
	return -1;
}

/* fonction I/O RS232 */

void OutRs232(char c)
{
while(!(SCSR&TDRE));
SCDR=c;
}

char InRs232()
{
	while(!(SCSR&RDRF));
	return (char)SCDR;
}

int write(int fd,char *buf,int nbytes)
{
	int i;
	for(i=0;i<nbytes;i++)
	{
/*		if (buf[i]=='\n')
			OutRs232(buf[i]);*/
		OutRs232(buf[i]);
	}
	return nbytes;
}

int read(int fd,char *buf,int nbytes)
{
	int i;
	for(i=0;i<nbytes;i++)
	{
		buf[i]=InRs232();
		if (buf[i]=='\n'||buf[i]=='\r')
		{
			buf[i]=0;
			break;
		}
	}
return i;
}

// gestion des ISR

void SetVect(int num,void (*p_func)())
{
struct f_isr *p_fisr;
int *p_vect;
p_vect=(int *)(0x800000+4*num);
p_fisr=(struct f_isr *)malloc(sizeof(struct f_isr));
p_fisr->sauv_reg=0x48E7FFFE;
p_fisr->c_jsr=0x4eb9;
p_fisr->adr_jsr=(int)p_func;
p_fisr->recup_reg=0x4cdf7fff;
p_fisr->retour=0x4e73;
p_fisr->old_vect=(int)*p_vect;
p_fisr->p_isr=p_fisr;
*p_vect=(int)p_fisr;
}

void RestoreVect(int num)
{
struct f_isr *p_fisr;
int *p_vect;
p_vect=(int *)(0x800000+4*num);
p_fisr=(struct f_isr *)*p_vect;
*p_vect=(int)p_fisr->old_vect;
free(p_fisr->p_isr);
}

void ValidIrq(short irq,int mode)
{

}

// gestion du simulateur d'E/S

void InitAff()
{
PortCDir=0xFF;
PortCPort=0x00;
}

void EcritAff(char valeur)
{
switch(valeur)
	{
	case 0 : PortCPort=AFF_0;
		 break;
	case 1 : PortCPort=AFF_1;
		break;
	case 2 : PortCPort=AFF_2;
		break;
	case 3 : PortCPort=AFF_3;
		break;
	case 4 : PortCPort=AFF_4;
		break;
	case 5 : PortCPort=AFF_5;
		break;
	case 6 : PortCPort=AFF_6;
		break;
	case 7 : PortCPort=AFF_7;
		break;
	case 8 : PortCPort=AFF_8;
		break;
	case 9 : PortCPort=AFF_9;
		break;
	}
}

void EcritPortA(char v)
{
short p_a=0x0000;
if (v&0x01)
	p_a|=1;
	else
	p_a|=2;
if (v&0x02)
	p_a|=0x04;
	else
	p_a|=0x08;
if (v&0x04)
	p_a|=0x10;
	else
	p_a|=0x20;
if (v&0x08)
	p_a|=0x40;
	else
	p_a|=0x80;
if (v&0x10)
	p_a|=0x100;
	else
	p_a|=0x200;
if (v&0x20)
	p_a|=0x400;
	else
	p_a|=0x800;
if (v&0x40)
	p_a|=0x1000;
	else
	p_a|=0x2000;
if (v&0x80)
	p_a|=0x4000;
	else
	p_a|=0x8000;
HSRR1=p_a;
}

void EcritPortB(char v)
{
short p_b=0x0000;
if (v&0x01)
	p_b|=1;
	else
	p_b|=2;
if (v&0x02)
	p_b|=0x04;
	else
	p_b|=0x08;
if (v&0x04)
	p_b|=0x10;
	else
	p_b|=0x20;
if (v&0x08)
	p_b|=0x40;
	else
	p_b|=0x80;
if (v&0x10)
	p_b|=0x100;
	else
	p_b|=0x200;
if (v&0x20)
	p_b|=0x400;
	else
	p_b|=0x800;
if (v&0x40)
	p_b|=0x1000;
	else
	p_b|=0x2000;
if (v&0x80)
	p_b|=0x4000;
	else
	p_b|=0x8000;
HSRR0=p_b;
}

void InitFeux()
{
CFSR0=(short)0x8888;
CFSR1=(short)0x8888;
CFSR2=(short)0x8888;
CFSR3=(short)0x8888;
HSQR0=(short)0;
HSQR1=(short)0;
CPR0=(short)0xFFFF;
CPR1=(short)0xFFFF;
CTRL_TPU0=(short)0x0003;
CTRL_TPU1=(short)0x0003;
CTRL_TPU2=(short)0x0003;
CTRL_TPU3=(short)0x0003;
CTRL_TPU4=(short)0x0003;
CTRL_TPU5=(short)0x0003;
CTRL_TPU6=(short)0x0003;
CTRL_TPU7=(short)0x0003;
CTRL_TPU8=(short)0x0003;
CTRL_TPU9=(short)0x0003;
CTRL_TPU10=(short)0x0003;
CTRL_TPU11=(short)0x0003;
CTRL_TPU12=(short)0x0003;
CTRL_TPU13=(short)0x0003;
CTRL_TPU14=(short)0x0003;
CTRL_TPU15=(short)0x0003;
PortCDir=0x07;
PortCPort=0x00;
EcritPortA(0x00);
EcritPortB(0x00);
}

void AllumeFeux(char n_f,char couleur)
{
short p_a=0;
short p_b=0;
short p_c=0;
switch(n_f)
	{
	case F1 : if (couleur&ROUGE)
			p_a|=0x0001;
		    if (couleur&ORANGE)
			p_a|=0x0004;
		    if (couleur&VERT)
			p_a|=0x0010;
		    HSRR1=p_a;
		    break;
	case F2 : if (couleur&ROUGE)
			p_a|=0x0040;
		    if (couleur&ORANGE)
			p_a|=0x0100;
		    if (couleur&VERT)
			p_a|=0x0400;
		    HSRR1=p_a;
		    break;
	case F3 : if (couleur&ROUGE)
			p_a|=0x1000;
		    if (couleur&ORANGE)
			p_a|=0x4000;
		    if (couleur&VERT)
			p_b|=0x0001;
		   HSRR1=p_a;
		   HSRR0=p_b;
		   break;
	case F4 : if (couleur&ROUGE)
			p_b|=0x0004;
		    if (couleur&ORANGE)
			p_b|=0x0010;
		    if (couleur&VERT)
			p_b|=0x0040;
		    HSRR0=p_b;
		    break;
	case F5 : if (couleur&ROUGE)
			p_b|=0x0100;
		    if (couleur&ORANGE)
			p_b|=0x0400;
		    if (couleur&VERT)
			p_b|=0x1000;
		    HSRR0=p_b;
		    break;
	case P1 : if (couleur&ROUGE)
			p_b|=0x4000;
		    HSRR0=p_b;
		    if (couleur&VERT)
			PortCPort|=0x01;
		    break;
	case P2 : if (couleur&ROUGE)
			PortCPort|=0x02;
		    if (couleur&VERT)
			PortCPort|=0x04;
		    break;	
	}
}

void EteindFeux(char n_f,char couleur)
{
short p_a=0;
short p_b=0;
short p_c=0;
switch(n_f)
	{
	case F1 : if (couleur&ROUGE)
			p_a|=0x0002;
		    if (couleur&ORANGE)
			p_a|=0x0008;
		    if (couleur&VERT)
			p_a|=0x0020;
		    HSRR1=p_a;
		    break;
	case F2 : if (couleur&ROUGE)
			p_a|=0x0080;
		    if (couleur&ORANGE)
			p_a|=0x0200;
		    if (couleur&VERT)
			p_a|=0x0800;
		    HSRR1=p_a;
		    break;
	case F3 : if (couleur&ROUGE)
			p_a|=0x2000;
		    if (couleur&ORANGE)
			p_a|=0x8000;
		    if (couleur&VERT)
			p_b|=0x0002;
		   HSRR1=p_a;
		   HSRR0=p_b;
		   break;
	case F4 : if (couleur&ROUGE)
			p_b|=0x0008;
		    if (couleur&ORANGE)
			p_b|=0x0020;
		    if (couleur&VERT)
			p_b|=0x0080;
		    HSRR0=p_b;
		    break;
	case F5 : if (couleur&ROUGE)
			p_b|=0x0200;
		    if (couleur&ORANGE)
			p_b|=0x0800;
		    if (couleur&VERT)
			p_b|=0x2000;
		    HSRR0=p_b;
		    break;
	case P1 : if (couleur&ROUGE)
			p_b|=0x8000;
		    HSRR0=p_b;
		    if (couleur&VERT&&PortCPort&0x01)
			PortCPort^=0x01;
		    break;
	case P2 : if (couleur&ROUGE&PortCPort&0x02)
			PortCPort^=0x02;
		    if (couleur&VERT&PortCPort&0x04)
			PortCPort^=0x04;
		    break;	
	}
}